@@ -91,15 +91,15 @@ sub validate_request {
 
   for my $p (@{$schema->{parameters} || []}) {
     my ($in, $name, $type) = @$p{qw(in name type)};
-    $type ||= $p->{schema}{type} if $p->{schema}; # v3
+    $type ||= $p->{schema}{type} if $p->{schema};    # v3
     my ($exists, $value) = (0, undef);
 
     if ($in eq 'body') {
-      $value = $self->_get_request_data($c, $in);
+      $value  = $self->_get_request_data($c, $in);
       $exists = length $value if defined $value;
     }
     elsif ($in eq 'formData' and $type eq 'file') {
-      $value = $self->_get_request_uploads($c, $name)->[-1];
+      $value  = $self->_get_request_uploads($c, $name)->[-1];
       $exists = $value ? 1 : 0;
     }
     else {
@@ -109,31 +109,17 @@ sub validate_request {
       $value  = $value->{$key};
     }
 
-    if (defined $value and ref $p->{items} eq 'HASH' and $p->{collectionFormat}) {
+    if (defined $value and $type eq 'array') {
       $value = $self->_coerce_by_collection_format($value, $p);
     }
 
     ($exists, $value) = (1, $p->{schema}{default})
-      if !$exists and $p->{schema} and exists $p->{schema}{default};
-    ($exists, $value) = (1, $p->{default})
-      if !$exists and exists $p->{default};
+      if !$exists
+      and $p->{schema}
+      and exists $p->{schema}{default};
+    ($exists, $value) = (1, $p->{default}) if !$exists and exists $p->{default};
 
-    if ($type and defined $value) {
-      if ($type ne 'array' and ref $value eq 'ARRAY') {
-        $value = $value->[-1];
-      }
-      if (($type eq 'integer' or $type eq 'number') and Scalar::Util::looks_like_number($value)) {
-        $value += 0;
-      }
-      elsif ($type eq 'boolean') {
-        if (!$value or $value =~ /^(?:false)$/) {
-          $value = Mojo::JSON->false;
-        }
-        elsif ($value =~ /^(?:1|true)$/) {
-          $value = Mojo::JSON->true;
-        }
-      }
-    }
+    $self->_coerce_input($type, $value);
 
     if (my @e = $self->_validate_request_value($p, $name => $value)) {
       push @errors, @e;
@@ -155,7 +141,7 @@ sub validate_response {
 
   if ($self->version eq '3') {
     my $accept = $self->_negotiate_accept_header($c, $res_schema);
-    return JSON::Validator::E('/' => "No responses rules defined for Accept $accept.")
+    return JSON::Validator::E('/' => "No responses rules defined for $accept.")
       unless $res_schema = $res_schema->{content}{$accept};
     $c->stash(openapi_negotiated_content_type => $accept);
   }
@@ -190,17 +176,69 @@ sub _build_formats {
   return $formats;
 }
 
-sub _coerce_by_collection_format {
-  my ($self, $data, $schema) = @_;
-  my $type = ($schema->{items} ? $schema->{items}{type} : $schema->{type}) || '';
+sub _coerce_input {
+  my ($self, $type) = @_;
+  return unless $type and defined $_[2];
 
-  if ($schema->{collectionFormat} eq 'multi') {
-    $data = [$data] unless ref $data eq 'ARRAY';
+  if ($type ne 'array' and ref $_[2] eq 'ARRAY') {
+    $_[2] = $_[2]->[-1];
+  }
+  if (($type eq 'integer' or $type eq 'number') and Scalar::Util::looks_like_number($_[2])) {
+    $_[2] += 0;
+  }
+  elsif ($type eq 'boolean') {
+    if (!$_[2] or $_[2] =~ /^(?:false)$/) {
+      $_[2] = Mojo::JSON->false;
+    }
+    elsif ($_[2] =~ /^(?:1|true)$/) {
+      $_[2] = Mojo::JSON->true;
+    }
+  }
+}
+
+sub _coerce_by_collection_format {
+  my ($self, $data, $p) = @_;
+
+  my $schema = $p->{schema}                                                  || $p;
+  my $type   = ($schema->{items} ? $schema->{items}{type} : $schema->{type}) || '';
+
+  my $collection_format = $p->{collectionFormat};
+  my $custom_re;
+
+  # support for v3 style / explode
+  if ($p->{style}) {
+    if ($p->{style} eq 'simple') {
+      $collection_format = 'csv';
+    }
+    elsif ($p->{style} eq 'label') {
+      $custom_re         = qr{\.};
+      $collection_format = $p->{explode} ? 'custom' : 'csv' if $data =~ s/^$custom_re//;
+    }
+    elsif ($p->{style} eq 'matrix') {
+      $custom_re         = qr{;\Q$p->{name}\E=};
+      $collection_format = $p->{explode} ? 'custom' : 'csv' if $data =~ s/^$custom_re//;
+    }
+    elsif ($p->{style} eq 'form') {
+      $collection_format = $p->{explode} ? 'multi' : 'csv';
+    }
+    elsif ($p->{style} eq 'spaceDelimited') {
+      $collection_format = $p->{explode} ? 'multi' : 'ssv';
+    }
+    elsif ($p->{style} eq 'pipeDelimited') {
+      $collection_format = $p->{explode} ? 'multi' : 'pipes';
+    }
+  }
+
+  return $data unless $collection_format;
+
+  if ($collection_format eq 'multi') {
+    $data  = [$data] unless ref $data eq 'ARRAY';
     @$data = map { $_ + 0 } @$data if $type eq 'integer' or $type eq 'number';
     return $data;
   }
 
-  my $re = $COLLECTION_RE{$schema->{collectionFormat}} || ',';
+  my $re = $collection_format eq 'custom' ? $custom_re : $COLLECTION_RE{$collection_format} || ',';
+
   my $single = ref $data eq 'ARRAY' ? 0 : ($data = [$data]);
 
   for my $i (0 .. @$data - 1) {
@@ -225,7 +263,7 @@ sub _get_request_data {
     return $c->match->stack->[-1];
   }
   elsif ($in eq 'formData') {
-    return $self->{cache}{$in} ||= $c->req->body_params->to_hash(1);
+    return $self->{cache}{$in} ||= $self->_get_request_form_and_uploads($c);
   }
   elsif ($in eq 'cookie') {
     return $self->{cache}{$in} ||= {map { ($_->name, $_->value) } @{$c->req->cookies}};
@@ -242,6 +280,13 @@ sub _get_request_data {
   }
 }
 
+sub _get_request_form_and_uploads {
+  my ($self, $c) = @_;
+  my $form = $c->req->body_params->to_hash(1);
+  push @{$form->{$_->name}}, $_->size ? 'x' : '' for @{$c->req->uploads};
+  return $form;
+}
+
 sub _get_request_uploads {
   my ($self, $c, $name) = @_;
   return $c->req->every_upload($name);
@@ -335,10 +380,14 @@ sub _validate_request_body {
   my $ct = $c->req->headers->content_type // '';
 
   $ct =~ s!;.*$!!;
-  if (my $schema = $body_schema->{content}{$ct}) {
+  if (my $content = $body_schema->{content}{$ct}) {
     my $body = $self->_get_request_data($c, $ct =~ /\bform\b/ ? 'formData' : 'body');
-    local $schema->{required} //= $body_schema->{required};
-    return $self->_validate_request_value($schema, body => $body);
+    local $content->{required} //= $body_schema->{required};
+    if (ref $content->{schema} eq 'HASH' and ref $body eq 'HASH') {
+      $self->_coerce_input($content->{schema}{properties}{$_}{type}, $body->{$_})
+        for keys %{$content->{schema}{properties} || {}};
+    }
+    return $self->_validate_request_value($content, body => $body);
   }
 
   return JSON::Validator::E('/' => "No requestBody rules defined for Content-Type $ct.") if $ct;
@@ -407,7 +456,7 @@ sub _validate_response_headers {
 sub _validate_type_array {
   my ($self, $data, $path, $schema) = @_;
 
-  if (ref $schema->{items} eq 'HASH' and $schema->{items}{collectionFormat}) {
+  if (ref $schema->{items} eq 'HASH' and ($schema->{items}{type} || '') eq 'array') {
     $data = $self->_coerce_by_collection_format($data, $schema->{items});
   }
 
