@@ -10,12 +10,13 @@ our %SIMPLE_HEADERS = map { (lc $_ => 1) }
   qw(Accept Accept-Language Content-Language Content-Type DPR Downlink Save-Data Viewport-Width Width);
 
 our %PREFLIGHTED_CONTENT_TYPES = %SIMPLE_CONTENT_TYPES;
-our %PREFLIGHTED_METHODS = map { ($_ => 1) } qw(CONNECT DELETE OPTIONS PATCH PUT TRACE);
+our %PREFLIGHTED_METHODS       = map { ($_ => 1) } qw(CONNECT DELETE OPTIONS PATCH PUT TRACE);
 
 my $X_RE = qr{^x-};
 
 sub register {
-  my ($self, $app, $openapi, $config) = @_;
+  my ($self, $app, $config) = @_;
+  my $openapi = $config->{openapi};
 
   if ($config->{add_preflighted_routes}) {
     $app->plugins->once(openapi_routes_added => sub { $self->_add_preflighted_routes($app, @_) });
@@ -40,7 +41,7 @@ sub register {
 
 sub _add_preflighted_routes {
   my ($self, $app, $openapi, $routes) = @_;
-  my $c = $app->build_controller;
+  my $c     = $app->build_controller;
   my $match = Mojolicious::Routes::Match->new(root => $app->routes);
 
   for my $route (@$routes) {
@@ -193,7 +194,10 @@ Mojolicious::Plugin::OpenAPI::Cors - OpenAPI plugin for Cross-Origin Resource Sh
 Set L</add_preflighted_routes> to 1, if you want "Preflighted" CORS requests to
 be sent to your already existing actions.
 
-  $app->plugin("OpenAPI" => {add_preflighted_routes => 1});
+  $app->plugin(OpenAPI => {add_preflighted_routes => 1, %openapi_parameters});
+
+See L<Mojolicious::Plugin::OpenAPI/register> for what
+C<%openapi_parameters> might contain.
 
 =head2 Simple exchange
 
@@ -356,7 +360,7 @@ Default value is 1800.
 
 =head2 openapi_cors_type
 
-This stash varible is available inside the callback passed on to
+This stash variable is available inside the callback passed on to
 L</openapi.cors_exchange>. It will be either "preflighted", "real" or "simple".
 "real" is the type that comes after "preflighted" when the actual request
 is sent to the server, but with "Origin" header set.
